@@ -24,6 +24,7 @@ local DEBUG = ngx.DEBUG
 local unpack = unpack
 local setmetatable = setmetatable
 local type = type
+local ipairs = ipairs
 
 
 local ok, new_tab = pcall(require, "table.new")
@@ -41,6 +42,7 @@ local IP6_ARPA = "ip6.arpa"
 local TYPE_A      = 1
 local TYPE_NS     = 2
 local TYPE_CNAME  = 5
+local TYPE_SOA    = 6
 local TYPE_PTR    = 12
 local TYPE_MX     = 15
 local TYPE_TXT    = 16
@@ -56,10 +58,11 @@ local SECTION_AR  = 3
 
 
 local _M = {
-    _VERSION    = '0.18',
+    _VERSION    = '0.19',
     TYPE_A      = TYPE_A,
     TYPE_NS     = TYPE_NS,
     TYPE_CNAME  = TYPE_CNAME,
+    TYPE_SOA    = TYPE_SOA,
     TYPE_PTR    = TYPE_PTR,
     TYPE_MX     = TYPE_MX,
     TYPE_TXT    = TYPE_TXT,
@@ -81,6 +84,7 @@ local resolver_errstrs = {
     "refused",          -- 5
 }
 
+local soa_int32_fields = { "serial", "refresh", "retry", "expire", "mininum" }
 
 local mt = { __index = _M }
 
@@ -346,12 +350,10 @@ local function parse_section(answers, section, buf, start_pos, size,
 
         -- print("class: ", class)
 
-        local ttl_bytes = { byte(buf, pos + 4, pos + 7) }
+        local byte_1, byte_2, byte_3, byte_4 = byte(buf, pos + 4, pos + 7)
 
-        -- print("ttl bytes: ", concat(ttl_bytes, " "))
-
-        local ttl = lshift(ttl_bytes[1], 24) + lshift(ttl_bytes[2], 16)
-                    + lshift(ttl_bytes[3], 8) + ttl_bytes[4]
+        local ttl = lshift(byte_1, 24) + lshift(byte_2, 16)
+                    + lshift(byte_3, 8) + byte_4
 
         -- print("ttl: ", ttl)
 
@@ -557,6 +559,27 @@ local function parse_section(answers, section, buf, start_pos, size,
 
             ans.ptrdname = name
 
+        elseif typ == TYPE_SOA then
+            local name, p = _decode_name(buf, pos)
+            if not name then
+                return nil, pos
+            end
+            ans.mname = name
+
+            pos = p
+            name, p = _decode_name(buf, pos)
+            if not name then
+                return nil, pos
+            end
+            ans.rname = name
+
+            for _, name in ipairs(soa_int32_fields) do
+                local byte_1, byte_2, byte_3, byte_4 = byte(buf, p, p + 3)
+                ans[name] = lshift(byte_1, 24) + lshift(byte_2, 16)
+                            + lshift(byte_3, 8) + byte_4
+                p = p + 4
+            end
+
         else
             -- for unknown types, just forward the raw value
 
@@ -681,6 +704,9 @@ local function parse_response(buf, id, opts)
     if opts then
         authority_section = opts.authority_section
         additional_section = opts.additional_section
+        if opts.qtype == TYPE_SOA then
+            authority_section = true
+        end
     end
 
     local err
